/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.extension.base;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.fortiss.tooling.kernel.ui.extension.IContextMenuContributor;
import org.fortiss.tooling.kernel.ui.extension.base.EObjectActionBase.EObjectActionFactory;
import org.fortiss.tooling.kernel.ui.extension.data.ContextMenuContextProvider;
import org.fortiss.tooling.kernel.ui.service.IContextMenuService;

/**
 * Base class for context menu contributors creating a sub-menu
 * {@link IMenuManager} with actions provided by actions factories
 * {@link EObjectActionFactory} returned by {@link #getActionFactories()}.
 * 
 * The class implements interface methods {@link #getMenuIcon()} and
 * {@link #getMenuId()} returning {@code null} by default. Classes providing an
 * own icon or id should override them.
 * 
 * @param <T>
 *            Indicates the type of selection elements.
 * 
 * @author trachtenherz
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating YELLOW Hash: C9D77819956A4B95AC544B63F2BADF2A
 */
public abstract class ContextMenuSubMenuContributorBase<T extends EObject>
		implements IContextMenuContributor {

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public final List<IContributionItem> getContributedItems(EObject selection,
			ContextMenuContextProvider contextProvider) {
		if (acceptSelection(selection, contextProvider)) {
			List<IContributionItem> contributionItems = new ArrayList<IContributionItem>();
			contributionItems
					.add(createMenu(createActionsForElement((T) selection)));
			return contributionItems;
		}
		return Collections.emptyList();
	}

	/** Creates action instances for the given selection. */
	protected Collection<Action> createActionsForElement(T selection) {
		ArrayList<Action> list = new ArrayList<Action>();
		for (EObjectActionFactory<? super T> factory : getActionFactories()) {
			list.add(factory.createAction(selection));
		}
		return list;
	}

	/**
	 * Creates a sub-menu {@link IMenuManager} containing specified actions.
	 */
	protected IMenuManager createMenu(Collection<Action> actions) {
		IMenuManager menu = new MenuManager(getMenuName(), getMenuIcon(),
				getMenuId());

		// Prepare menu structure hints
		MenuStructureHints[] hints;
		if (getMenuStructureHints() != null) {
			hints = Arrays.copyOf(getMenuStructureHints(), actions.size());
		} else {
			hints = new MenuStructureHints[actions.size()];
			Arrays.fill(hints, null);
		}
		// Now the hints array contains all elements from
		// getMenuStructureHints(). It
		// also has at least as many elements as the actions collection. If
		// getMenuStructureHints() has less elements (or is null),
		// than all other elements of the hints array are null.
		int i = 0;
		for (Action a : actions) {
			if (hints[i] == MenuStructureHints.SEPARATOR_BEFORE) {
				menu.add(new Separator());
			}
			menu.add(a);
			if (hints[i] == MenuStructureHints.SEPARATOR_AFTER) {
				menu.add(new Separator());
			}

			++i;
		}
		return menu;
	}

	/** {@inheritDoc} */
	@Override
	public String getMenuSectionID() {
		return IContextMenuService.BOTTOM_MOST_MENU_SECTION_ID;
	}

	/**
	 * Returns {@code true} if the selected element is acceptable. This method
	 * must also ensure that {@code selection} is an instance of {@code T}
	 */
	protected abstract boolean acceptSelection(EObject selection,
			ContextMenuContextProvider contextProvider);

	/** The name of the sub-menu */
	protected abstract String getMenuName();

	/** Returns the action factories for the actions to be shown in the menu */
	protected abstract Collection<EObjectActionFactory<? super T>> getActionFactories();

	/**
	 * Constants for menu structure hints.
	 * 
	 * @see ContextMenuSubMenuContributorBase#getMenuStructureHints()
	 */
	protected static enum MenuStructureHints {
		/** Insert a separator before current menu item. */
		SEPARATOR_BEFORE,
		/** Insert a separator after current menu item. */
		SEPARATOR_AFTER;
		// Further future hints may be e.g.: Begin new sub-menu, End new
		// sub-menu
	}

	/**
	 * Returns additional informations about the structure of the menu. If
	 * {@code null}, the actions are linearly shown in a menu.
	 * 
	 * @see #getActionFactories()
	 * @see #createMenu(Collection)
	 */
	protected MenuStructureHints[] getMenuStructureHints() {
		return null;
	}

	/** The image for the sub-menu */
	protected ImageDescriptor getMenuIcon() {
		return null;
	}

	/** The Id for the sub-menu */
	protected String getMenuId() {
		return null;
	}

}